<?php

namespace Litecms\Practice\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Practice\Http\Requests\PracticeRequest;
use Litecms\Practice\Interfaces\PracticeRepositoryInterface;
use Litecms\Practice\Models\Practice;

/**
 * Resource controller class for practice.
 */
class PracticeResourceController extends BaseController
{

    /**
     * Initialize practice resource controller.
     *
     * @param type PracticeRepositoryInterface $practice
     *
     * @return null
     */
    public function __construct(PracticeRepositoryInterface $practice)
    {
        parent::__construct();
        $this->repository = $practice;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Practice\Repositories\Criteria\PracticeResourceCriteria::class);
    }

    /**
     * Display a list of practice.
     *
     * @return Response
     */
    public function index(PracticeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Practice\Repositories\Presenter\PracticePresenter::class)
                ->$function();
        }

        $practices = $this->repository->paginate();

        return $this->response->title(trans('practice::practice.names'))
            ->view('practice::practice.index', true)
            ->data(compact('practices'))
            ->output();
    }

    /**
     * Display practice.
     *
     * @param Request $request
     * @param Model   $practice
     *
     * @return Response
     */
    public function show(PracticeRequest $request, Practice $practice)
    {

        if ($practice->exists) {
            $view = 'practice::practice.show';
        } else {
            $view = 'practice::practice.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('practice::practice.name'))
            ->data(compact('practice'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new practice.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PracticeRequest $request)
    {

        $practice = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('practice::practice.name')) 
            ->view('practice::practice.create', true) 
            ->data(compact('practice'))
            ->output();
    }

    /**
     * Create new practice.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PracticeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $practice                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('practice::practice.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('practice/practice/' . $practice->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/practice/practice'))
                ->redirect();
        }

    }

    /**
     * Show practice for editing.
     *
     * @param Request $request
     * @param Model   $practice
     *
     * @return Response
     */
    public function edit(PracticeRequest $request, Practice $practice)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('practice::practice.name'))
            ->view('practice::practice.edit', true)
            ->data(compact('practice'))
            ->output();
    }

    /**
     * Update the practice.
     *
     * @param Request $request
     * @param Model   $practice
     *
     * @return Response
     */
    public function update(PracticeRequest $request, Practice $practice)
    {
        try {
            $attributes = $request->all();

            $practice->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('practice::practice.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('practice/practice/' . $practice->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('practice/practice/' . $practice->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the practice.
     *
     * @param Model   $practice
     *
     * @return Response
     */
    public function destroy(PracticeRequest $request, Practice $practice)
    {
        try {

            $practice->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('practice::practice.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('practice/practice/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('practice/practice/' . $practice->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple practice.
     *
     * @param Model   $practice
     *
     * @return Response
     */
    public function delete(PracticeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('practice::practice.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('practice/practice'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/practice/practice'))
                ->redirect();
        }

    }

    /**
     * Restore deleted practices.
     *
     * @param Model   $practice
     *
     * @return Response
     */
    public function restore(PracticeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('practice::practice.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/practice/practice'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/practice/practice/'))
                ->redirect();
        }

    }

}
